package WeddingGrid;

import repast.simphony.context.Context;
import repast.simphony.context.space.continuous.ContinuousSpaceFactory;
import repast.simphony.context.space.continuous.ContinuousSpaceFactoryFinder;
import repast.simphony.context.space.graph.NetworkBuilder;
import repast.simphony.context.space.grid.GridFactory;
import repast.simphony.context.space.grid.GridFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.SimpleCartesianAdder;
import repast.simphony.space.continuous.WrapAroundBorders;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridBuilderParameters;
import repast.simphony.space.grid.SimpleGridAdder;

public class Builder implements ContextBuilder<Object> {
	private TheWorld theWorld;
	ContinuousSpace<Object> space;
	Grid<Object> grid;
	StatisticCollector stc;
	Context<Object> context;
	int width, height;
	
	public Builder() {
		stc = new StatisticCollector();
	}

	/**
	 * Create a new world based on the provided parameters. If the height of the space is 0, 
	 * the world is a tube (ie. Billari's model). If not, the world is a grid.
	 * 
	 * @see		TheWorld, TheGrid, TheRing
	 */
	@Override
	public Context<Object> build(Context<Object> context) {
		this.context = context;
		context.setId("Wedding Grid");
		
		int width = (Integer) RunEnvironment.getInstance().getParameters().getValue("spaceWidth");
		int height = (Integer) RunEnvironment.getInstance().getParameters().getValue("spaceHeight");
		
		this.width = width; 
		this.height = height;
		
		if (height == 0) height = (Integer) RunEnvironment.getInstance().getParameters().getValue("maxAge");
		if (height == 0) height = Data.defaultMaxAge;
		
		ContinuousSpaceFactory spaceFactory = ContinuousSpaceFactoryFinder.createContinuousSpaceFactory(null);
		space = spaceFactory.createContinuousSpace("space", context, new SimpleCartesianAdder<Object>(), new WrapAroundBorders(), width, height);
		
		GridFactory gridFactory = GridFactoryFinder.createGridFactory(null);
		grid = gridFactory.createGrid("grid", context, new GridBuilderParameters<Object>(new repast.simphony.space.grid.WrapAroundBorders(), new SimpleGridAdder<Object>(), true, width, height));
		
		NetworkBuilder<Object> netBuilder = new NetworkBuilder<Object>("Married network", context, false);
		netBuilder.buildNetwork();

		if (this.height > 0) theWorld = new TheGrid(context, space, grid, stc);
		else theWorld = new TheRing(context, space, grid, stc);
		context.add(theWorld);
		context.add(this);
		
		double endTick = 0.9 + (Integer) RunEnvironment.getInstance().getParameters().getValue("numOfTick");
		
		// Initiate various variables and classes for statistics collection, dependent on type of run.
		
		stc.setRunType();
		
		if (RunEnvironment.getInstance().isBatch()) {
			RunEnvironment.getInstance().endAt(endTick);
			if (stc.getCount() == 1) {
				stc.setTime();
				stc.newHazardRecord();
				stc.newDecadeRecord();
			}
		} else {
			stc.resetCount();
			stc.setTime();
			stc.newHazardRecord();
			stc.newDecadeRecord();
		}
		stc.newHumanRecord();
		stc.newYearlyRecord();
				
		return context;
	}

	public int size() {
		return context.getObjects(Human.class).size();
	}
	
	public int femaleSize() {
		return context.getObjects(Female.class).size();
	}
	
	public int maleSize() {
		return context.getObjects(Male.class).size();
	}
	
}