package WeddingGrid;

import java.util.ArrayList;
import java.util.Observable;
import java.util.Observer;

import repast.simphony.engine.environment.RunEnvironment;


/*
 * Class to record cohort-wise hazard rates 
 * 
 */

public class Cohort implements Observer {
	
	protected ArrayList<Human> agents;
	protected double[] hazardRate;
	protected int count, married, newlyMarried, marriageable;
	protected int maxAge;
	private boolean finished;
	
	public Cohort(ArrayList<Human> agents) {
		this.agents = agents;
		maxAge = (Integer) RunEnvironment.getInstance().getParameters().getValue("maxAge");
		if (maxAge == 0) maxAge = Data.defaultMaxAge;
		hazardRate = new double[maxAge + 1];
		count = 0;
		married = 0;
		newlyMarried = 0;
		marriageable = agents.size();
		finished = false;
		
		for (Human human : agents) human.addObserver(this);
	}
	
	/**
	 * Calculate the hazard rate for the current age of this cohort.
	 */
	public void calculateHazardRate() {
		if (finished) return;
		count++;
		if (count <= maxAge + 1) {
			if ((marriageable == 0) || (count == maxAge + 1)) {
				finished = true;
				return;
			}
			hazardRate[count] = ((double) (newlyMarried) / marriageable);
			married = married + newlyMarried;
			newlyMarried = 0;
			marriageable = agents.size() - married;
		}
	}
	
	/**
	 * Check if the calculation for this cohort has finished
	 * 
	 * @return		True if all agents in this cohort have either got married of died. False otherwise.
	 */
	public boolean isFinished() {
		return finished;
	}
	
	/**
	 * Return the hazard rates.
	 * 
	 * @return		An array contain the hazard rates with respect to ages.
	 */
	public double[] getHazardRates() {
		return hazardRate;
	}

	/**
	 * Update the cohort when when there is a change in an agent of this cohort.
	 * 
	 * @param	human	the agent that called Observable.notifyObservers().
	 * @param	obj		the object passed as the argument.
	 * 
	 * @see		Human, TheWorld.update(Observable human, Object obj)
	 */
	public void update(Observable human, Object obj) {
		if (obj instanceof String) {
			if (((String) obj).equals("Died")) {
				if (((Human) human).isMarried()) {
					married--;
				}
				agents.remove((Human) human);
			} else if (((String) obj).equals("Got married")) {
				newlyMarried++;
			}
		}
	}
}
