package WeddingGrid;

import java.util.ArrayList;

/*
 * Class contains the empirical and projected fertility and mortality rates. 
 * those paths and arrays labeled "new" are used in latest production of results.
 * 'old' and 'default' are now deprecated  
 *
 */

public class Data {
	
	public static final int defaultMaxAge = 150;
	
	protected static Double[][] deathRateEQ, fertRateEQ;
	protected static Double[][] maleDeathRate, femaleDeathRate, fertRate,birthProps;
	
	public static final String defaultDeathRatePath = "Death.rate.csv", defaultFertRatePath = "Fertility.rate.csv",
							   maleDeathRatePath = "Death.rate.new.male.csv", femaleDeathRatePath = "Death.rate.new.female.csv",
							   newFertRatePath = "Fertility.rate.new.csv", birthPropPath="birthprops.csv";
	
	static {
		CSVHandler dataReader = new CSVHandler();
		dataReader.setPath(defaultDeathRatePath);
		try {
			dataReader.openFile(true);
		
			String[][] strl = dataReader.read();
			if (strl.length > 0) {
				ArrayList<Double[]> arrs = new ArrayList<Double[]>();
				
				for (String[] strs : strl) {
					ArrayList<Double> arr = new ArrayList<Double>(); 
					for (String str : strs)	arr.add(Double.parseDouble(str));
					arrs.add(arr.toArray(new Double[1]));
				}		
				deathRateEQ = arrs.toArray(new Double[1][1]);
				dataReader.closeFile();
			}
		} catch (Exception e) {e.printStackTrace();}
		
		dataReader.setPath(defaultFertRatePath);
		try {
			dataReader.openFile(true);
			String[][] strl = dataReader.read();
			if (strl.length > 0) {
				ArrayList<Double[]> arrs = new ArrayList<Double[]>(); 
				for (String[] strs : strl) {	
					ArrayList<Double> arr = new ArrayList<Double>(); 
					for (String str : strs)	arr.add(Double.parseDouble(str));
					arrs.add(arr.toArray(new Double[1]));
				}		
				fertRateEQ = arrs.toArray(new Double[1][1]);
				dataReader.closeFile();
			}
		} catch (Exception e) {e.printStackTrace();}
		
		dataReader.setPath(maleDeathRatePath);
		try {
			dataReader.openFile(true);
			String[][] strl = dataReader.read();
			if (strl.length > 0) {
				ArrayList<Double[]> arrs = new ArrayList<Double[]>(); 
				for (String[] strs : strl) {	
					ArrayList<Double> arr = new ArrayList<Double>(); 
					for (String str : strs)	arr.add(Double.parseDouble(str));
					arrs.add(arr.toArray(new Double[1]));
				}		
				maleDeathRate = arrs.toArray(new Double[1][1]);
				dataReader.closeFile();
			}
		} catch (Exception e) {e.printStackTrace();}
		
		dataReader.setPath(femaleDeathRatePath);
		try {
			dataReader.openFile(true);
			String[][] strl = dataReader.read();
			if (strl.length > 0) {
				ArrayList<Double[]> arrs = new ArrayList<Double[]>(); 
				for (String[] strs : strl) {	
					ArrayList<Double> arr = new ArrayList<Double>();
					for (String str : strs)	arr.add(Double.parseDouble(str));
					arrs.add(arr.toArray(new Double[1]));
				}		
				femaleDeathRate = arrs.toArray(new Double[1][1]);
				dataReader.closeFile();
			}
		} catch (Exception e) {e.printStackTrace();}
		
		dataReader.setPath(newFertRatePath);
		try {
			dataReader.openFile(true);
			String[][] strl = dataReader.read();
			if (strl.length > 0) {
				ArrayList<Double[]> arrs = new ArrayList<Double[]>(); 
				for (String[] strs : strl) {	
					ArrayList<Double> arr = new ArrayList<Double>(); 
					for (String str : strs)	arr.add(Double.parseDouble(str));
					arrs.add(arr.toArray(new Double[1]));
				}		
				fertRate = arrs.toArray(new Double[1][1]);
				dataReader.closeFile();
			}
		} catch (Exception e) {e.printStackTrace();}
		dataReader.setPath(birthPropPath);
		try {
			dataReader.openFile(true);
			String[][] strl = dataReader.read();
			if (strl.length > 0) {
				ArrayList<Double[]> arrs = new ArrayList<Double[]>(); 
				for (String[] strs : strl) {	
					ArrayList<Double> arr = new ArrayList<Double>(); 
					for (String str : strs)	arr.add(Double.parseDouble(str));
					arrs.add(arr.toArray(new Double[1]));
				}		
				birthProps = arrs.toArray(new Double[1][1]);
				dataReader.closeFile();
			}
		} catch (Exception e) {e.printStackTrace();}
		
		
	}
	
	public static double marriedRate(Human human) {
		/* married proportions by age for 1950 based on UK 
		 * census data for setting up  simulation 
		 */
		double r;
		int a = human.age;
		if (a < 16) return 0;
		if (human instanceof Male) {
			if (a < 20) r = 0.005;
			else if (a < 25) r = 0.23715;
			else if (a < 30) r = 0.64606;	
			else if (a < 35) r = 0.79882;	
			else if (a < 40) r = 0.85139;	
			else if (a < 45) r = 0.87194;	
			else if (a < 50) r = 0.87752;	
			else if (a < 55) r = 0.87622;	
			else if (a < 60) r = 0.86737;	
			else if (a < 65) r = 0.8304;	
			else if (a < 70) r = 0.77055;	
			else if (a < 75) r = 0.68214;	
			else if (a < 80) r = 0.56818;	
			else if (a < 85) r = 0.44349;	
			else if (a < 90) r = 0.32709;	
			else if (a < 95) r = 0.23405;	
			else r = 0.20457;	
		}
		else {
			if (a < 20) r = 0.04415;	
			else if (a < 25) r = 0.47982;	
			else if (a < 30) r = 0.76986;	
			else if (a < 35) r = 0.82679;	
			else if (a < 40) r = 0.83065;	
			else if (a < 45) r = 0.81015;	
			else if (a < 50) r = 0.77958;	
			else if (a < 55) r = 0.73744;	
			else if (a < 60) r = 0.66839;	
			else if (a < 65) r = 0.57439;	
			else if (a < 70) r = 0.47686;	
			else if (a < 75) r = 0.36833;	
			else if (a < 80) r = 0.25725;	
			else if (a < 85) r = 0.15351;	
			else if (a < 90) r = 0.08484;	
			else if (a < 95) r = 0.04688;	
			else r = 0.05004;
		}
		
		return r;
	}
	
	public static double deathRate(Human human) {
		return newDeathRate(human);	
	}
	
	public static double fertRate(Human human) {
		return newFertRate(human);
	}
	
	public static double getBirthProp(Human human) {
		return birthProp(human);
	}
	
	
	private static double oldDeathRate(Human human) {
		int year = human.yearOfBirth + human.age - 1950;
		int age = human.age < 109 ? human.age : 109;
		
		double a = age > 0 ? 0.5 : ((human instanceof Male) ? 0.335 : 0.330);
		double mr = deathRateEQ[age][0] * year + deathRateEQ[age][1];
		double q = (Math.exp(mr) / (1 + (1 - a) * Math.exp(mr)));
		
		return q;
	}
	
	private static double newDeathRate(Human human) {
		int year = human.yearOfBirth + human.age - 1950;
		int age = human.age < 109 ? human.age : 109;
		
		if (human instanceof Male) return maleDeathRate[age][year];
		else return femaleDeathRate[age][year];
	}
	
	private static double oldFertRate(Human human) {
		int year = human.yearOfBirth + human.age - 1950;
		int age = human.age < 50 ? human.age : 50;
		if (age == 50) return 0;
		
		double a = 0.5;
		double fr = fertRateEQ[age][0] * year + fertRateEQ[age][1];
		double q = (Math.exp(fr) / (1 + (1 - a) * Math.exp(fr)));
		
		return q;
	}
	
	private static double newFertRate(Human human) {
		int year = human.yearOfBirth + human.age - 1950;
		int age = human.age < 50 ? human.age : 50;
		if (age == 50) return 0;
		
		return fertRate[age - 15][year];
	}
	
	private static double birthProp(Human human) {
		int year = human.yearOfBirth + human.age - 1950;
		int age = human.age < 50 ? human.age : 50;
		if (age == 50) return 0;
		
		return birthProps[age - 14][year]; /// check the indexing here!!
	}
	
}