package WeddingGrid;

import java.util.ArrayList;

import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.graph.Network;
import repast.simphony.space.grid.Grid;
import repast.simphony.util.ContextUtils;

/**
 * A class that contain methods for Female agents only.
 * 
 * @author James
 *
 */

public class Female extends Human {
	double marriedPercentage;
	public int parity=0; 
	public Female(Context<Object> context, ContinuousSpace<Object> space, Grid<Object> grid, int yearOfBirth, int age, boolean isYAxisAge) {
		super(context, space, grid, yearOfBirth, age, isYAxisAge);
		display = new FemaleNormalStyle(this);
		context.add(display);
	}

	
	/**
	 * Find potential partners for the agent
	 */
	@Override
	public void findPotentialPartners() {
		potentialPartners = new ArrayList<Human>();
		Context<Object> context = ContextUtils.getContext(this);
		for (Object obj : context.getObjects(Male.class)) {
			if (isAPotentialPartner((Human) obj)) {
				potentialPartners.add((Human) obj);
			}			
		}
	}
	
	/**
	 * An extension of the function exclusively for female agents. Calculate fertility rate and give birth if appropriate
	 * @see		Human.marriageActivities()
	 */
	@Override
	public void marriageActivities() {
		super.marriageActivities();
		double r = RandomHelper.nextDoubleFromTo(0, 1);
		if (isMarried() & this.age< 49) {
			double rate = (Data.fertRate(this) * Data.getBirthProp(this))/marriedPercentage; 
			if (r <= rate) giveBirth();
		}
		else if (isMarried()==false & this.age>15 & this.age<49){
			double rate = Data.fertRate(this) * (1.0-Data.getBirthProp(this))/(1.0-marriedPercentage);
			if (r <= rate) giveBirth();

		}
	}
	
	

	/**
	 * Update the percentage of married Female agents.
	 * 
	 * @param 	marriedPercentage the percentage of married Female agents
	 * 
	 * @see 	TheWorld.eachyear().
	 */
	public void updateMarriedPercentage(double marriedPercentage) {
		this.marriedPercentage = marriedPercentage;
	}
	

	
	/**
	 * Add a newborn child to the world. The location of the child is determined by the type of the world.
	 * Notify the world.
	 * 
	 * @see		TheWorld
	 */
	public void giveBirth() {
		double i = RandomHelper.nextDoubleFromTo(0, 2.048);
		Human child;
		Context<Object> context = ContextUtils.getContext(this);
		if (i < 1.048) child = new Male(context, space, grid, yearOfBirth + age, 0, isYAxisAge);
		else child = new Female(context, space, grid, yearOfBirth + age, 0, isYAxisAge);
		children.add(child);
		if (partner != null) partner.children.add(child);
		
		NdPoint pt = space.getLocation(display);
		double yLocation = isYAxisAge ? 0 : RandomHelper.nextDoubleFromTo(pt.getY() - sd * 2, pt.getY() + sd * 2);
		
		space.moveTo(child.display, RandomHelper.nextDoubleFromTo(pt.getX() - sd * 2, pt.getX() + sd * 2), yLocation);
		pt = space.getLocation(child.display);
		grid.moveTo(child.display, (int) pt.getX(), (int) pt.getY());
		parity = parity + 1;
		setChanged();
		notifyObservers(child);	
	}
	
	/**
	 * Return the gender of the agent
	 * 
	 * @return 		The gender
	 */
	public String getGender() {
		return "Female";
	}
	
	public boolean getHealthStatus() {
		return isIll;
	}

	
	/**
	 * Determine whether agent becomes ill. 
	 * Probability based on agents age 
	 * and  ageScaling, mainCareProb, baseCareProb parameters
	 * 
	 */
	@Override
	public void careTransition() {
		
		double ageScaling = (Double) RunEnvironment.getInstance().getParameters().getValue("ageScaling");
		double mainCareProb = (Double) RunEnvironment.getInstance().getParameters().getValue("mainCareProb");
		double baseCareProb = (Double) RunEnvironment.getInstance().getParameters().getValue("baseCareProb");
		double ageCareProb = ((Math.exp((double) age / (ageScaling - 1.0)) * (mainCareProb - 0.0001)));
		
		double careProb = baseCareProb + ageCareProb;

		if (Math.random() < careProb) {
			Context<Object> context = ContextUtils.getContext(this);
			Network<Object> net = (Network<Object>)context.getProjection("Married network");
			Object adj = net.getRandomAdjacent(display);
			NdPoint pt = space.getLocation(display);
			context.remove(display);
			display = new FemaleInNeedStyle(this);
			context.add(display);
			space.moveTo(display, pt.getX(), pt.getY());
			grid.moveTo(display, (int) pt.getX(), (int) pt.getY());
			
			if (adj != null) {
				net.addEdge(display, adj);
				net.addEdge(adj, display);
			}
			isIll = true;
			setChanged();
			notifyObservers("ill");
		}
	}
}
