package WeddingGrid;
import java.util.ArrayList;
import java.util.Observable;

import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.graph.Network;
import repast.simphony.space.grid.Grid;
import repast.simphony.util.ContextUtils;
import repast.simphony.util.SimUtilities;

/**
 * A class that contain common behaviors of both Male and Female type agents.
 *  
 * @author James
 *
 */

public abstract class Human extends Observable {

	protected ContinuousSpace<Object> space;
	protected Grid<Object> grid;
	protected int ID, yearOfBirth, yearOfDeath, yearOfMarriage, age;
	protected double midpoint, ageInterval, socialPressure, distToSearch, ageToSearch, pom, distanceToCarer;
	protected int marDur, type;
	protected int maxAge, minPop, initPop;
	protected boolean isYAxisAge, isIll;
	protected ArrayList<Human> relots, potentialPartners, children;
	protected Human partner;
	protected HumanStyle display;
	protected double m;

	protected static double alpha = (Double) RunEnvironment.getInstance().getParameters().getValue("alpha");
	protected static double beta = (Double) RunEnvironment.getInstance().getParameters().getValue("beta"), sd = (Double) RunEnvironment.getInstance().getParameters().getValue("sd"), 
			gamma = 2, maxAgeInterval = 2;
	protected static double aiConstant =((Double) RunEnvironment.getInstance().getParameters().getValue("AIConstant"));
	
	public Human(Context<Object> context, ContinuousSpace<Object> space, Grid<Object> grid, int yearOfBirth, int age, boolean isYAxisAge) {
		this.space = space;
		this.grid = grid;

		this.yearOfBirth = yearOfBirth;
		this.yearOfDeath = 0;
		this.distanceToCarer = 999;
		this.age = age;
		this.marDur = 0;
		this.isYAxisAge = isYAxisAge;

		this.initPop = ((Integer) RunEnvironment.getInstance().getParameters().getValue("initPop"));
		this.minPop = ((Integer) RunEnvironment.getInstance().getParameters().getValue("minPop"));
		this.maxAge = ((Integer) RunEnvironment.getInstance().getParameters().getValue("maxAge"));
		

		children = new ArrayList<Human>();

		context.add(this);
	}

	/**
	 * Initiate the attributes of a human based on Billari's model.
	 */
	public void initHuman() {
		type = RandomHelper.nextIntFromTo(1, 5);
		double g = RandomHelper.nextDoubleFromTo(0, gamma);
		switch (type) {
			case 1: midpoint = -g; break;
			case 2: midpoint = -g/2; break;
			case 4: midpoint = g/2; break;
			case 5: midpoint = g; break;
			default: midpoint = 0;
		}

		ageInterval = RandomHelper.nextDoubleFromTo(0, maxAgeInterval);
		alpha = (Double) RunEnvironment.getInstance().getParameters().getValue("alpha");
		beta = (Double) RunEnvironment.getInstance().getParameters().getValue("beta");
		sd = (Double) RunEnvironment.getInstance().getParameters().getValue("sd");
		aiConstant =(Double) RunEnvironment.getInstance().getParameters().getValue("AIConstant");
	}

	/**
	 * Checking if the agent should die. If yes, notify the world. 
	 * If not, increase the age and the marriage duration if the agent is married.
	 * Move the agent along the age axis if the model is a ring not a grid
	 * 
	 * @see		TheWorld
	 */
	@ScheduledMethod(start = 1, interval = 1)
	public void ageing() {
		if (death()) {
			yearOfDeath = yearOfBirth + age;
			if (marDur > 0) yearOfMarriage = yearOfDeath - marDur + 1;
			setChanged();
			notifyObservers("Died");
			Context<Object> context = ContextUtils.getContext(this);
			context.remove(this);
			context.remove(display);
		} else {
			age++;
			if ((age >= 16) && (relots == null)){
				initHuman();
			}
			if (isYAxisAge) yearlyMovement();
			if (isMarried()) marDur++;
			setChanged();
			notifyObservers();
		}
	}
	
	/**
	 * Move the agent up along the Y axis. Only relevant when using Billar's ring.
	 */
	public void yearlyMovement() {
		grid.moveByDisplacement(display, 0, 1);
		space.moveByDisplacement(display, 0, 1);
	}

	/**
	 * Choose new relevant others and calculate new social pressure.
	 */
	@ScheduledMethod(start = 1.2, interval = 1)
	public void yearlyActivities() {
		if (age > 15) {
			if (!isMarried()) {
				chooseRelevantOthers();
				calculateSocialPressure();
			} 
		}
	}
	
	
	/**
	 * If the agent is matured and hasn't got married, attempt to find a partner.
	 * 
	 * @see		Female.marriageActivities()
	 */
	@ScheduledMethod(start = 1.25, interval = 1)
	public void marriageActivities() {
		if (age > 15) {
			if (!isMarried()) {
				areaForPotentialPartners();
				findPotentialPartners();
				choosePartner();
			}
		}
		if (isIll == false) careTransition();
	}

	/**
	 * Check if the agent should die. The data is taken from Data class.
	 * 
	 * @return 	The fate of the agent.
	 * 
	 * @see 	Data
	 */
	public boolean death() {
		if (maxAge > 0) {
			if (age == maxAge) return true;
			else return false;
		} else {
			if  ( age == Data.defaultMaxAge) return true;  
			else if (Math.random() <= Data.deathRate(this)) return true;
			else return false;
		}
	}
	
	
	/**
	 * Find and choose relevant others.
	 */
	public void chooseRelevantOthers() {
		relots = new ArrayList<Human>();

		Context<Object> context = ContextUtils.getContext(this);
		for (Object obj : context.getObjects(Human.class)) {
			if (isARelevantOther((Human) obj)) {
				double i = RandomHelper.nextDoubleFromTo(0, 1);
				if (i < 0.7) {
					relots.add((Human) obj); 
				}
			}			
		}
	}

	/**
	 * Calculate social pressure based on Billari's model.
	 */
	public void calculateSocialPressure() {
		int numOfMarried = 0;
		for (Human relevant : relots) {
			if (relevant.isMarried()) numOfMarried++;
		}

		pom = relots.size() == 0 ? 0 : (double) numOfMarried / relots.size();
		if (pom == 0) socialPressure = 0.05;
		else socialPressure = Math.exp(beta * (pom - alpha)) / (1 + Math.exp(beta * (pom - alpha)));
		//alternate constant SP scenario
		//else if (pom > 0 && pom < 0.1) socialPressure = 1.5 * pom + 0.05;
		//else if (pom > 0.1) socialPressure = 0.2;
	}

	 /**
     * Calculate the area to search for potential partners.
     * Area is a function of the age of the agent, the social pressure and size of the current population
     */
    public void areaForPotentialPartners() {
    	Context<Object> context = ContextUtils.getContext(this);
    	double distance =(double) (space.getDimensions().getWidth() / 2);
    	double ai = 0.0;
    	int c = 25;
    	
    	//billari-based age-influence
    	if (age < 16 || age > 116) ai = 0;
    	else if (age >= 16 && age <= 20) ai = 0.2 * age - 3.1;
        else if (age >= 21 && age <= 33) ai = 0.9;
        else if (age >= 34 && age <= 38) ai = -0.1 * age + 4.3;
        else if (age >= 39 && age <= 59) ai = 0.5;
        else if (age >= 60 && age <= 64) ai = -0.1 * age + 6.5;
        else if (age >= 65 && age <= 116) ai = 0.1;

    	if (isYAxisAge){ distToSearch =socialPressure * ((double)minPop / context.getObjects(Human.class).size()) * ai * distance;}
    	else {distToSearch = java.lang.Math.sqrt( socialPressure * ((double) minPop / context.getObjects(Human.class).size()) * ai) * distance;}

    	ageToSearch = socialPressure * ai * c;
    }

	/**
	 * Find potential partners for the agent. To be implemented by subclass
	 */
	public abstract void findPotentialPartners();

	/**
	 * Choose a partner among potential partners.
	 */
	public void choosePartner() {
		SimUtilities.shuffle(potentialPartners, RandomHelper.getUniform());
		for (Human h : potentialPartners) {
			if (h.isAPotentialPartner(this)) {
				getMarried(h);
				h.getMarried(this);
				if (!isYAxisAge) moveToNewHome((double) relots.size() / partner.relots.size());
				break;
			}
		}
	}

	/**
	 * Set marriage duration of the agent.
	 * 
	 * @param marDur	The marriage duration.
	 */
	public void setMarDur(int marDur) {
		this.marDur = marDur;
	}

	/**
	 * Set ID for the agent.
	 * 
	 * @param 	ID	The ID to be set.
	 */
	public void setID(int ID) {
		this.ID = ID;
	}
	
	public double getX() {
		NdPoint pt = space.getLocation(display);
		return pt.getX();
	}
	
	public double getY() {
		NdPoint pt = space.getLocation(display);
		return pt.getY();
	}

	/**
	 * Set the spouse of the agent when get married
	 * 
	 * @param 	partner	The spouse.
	 */
	public void getMarried(Human partner) {
		this.partner = partner;
		marDur = 1;

		Context<Object> context = ContextUtils.getContext(this);
		Network<Object> net = (Network<Object>)context.getProjection("Married network");
		net.addEdge(display, partner.display);

		setChanged();
		notifyObservers("Got married");
	}

	/**
	 * Move the agent to a new location when get married. Only relevant when using the grid.
	 */
	public void moveToNewHome(double ratio) {
		NdPoint pt = space.getLocation(display);
		NdPoint otherPt = space.getLocation(partner.display);
		double distantX = otherPt.getX() - pt.getX();
		double distantY = otherPt.getY() - pt.getY();
		double width = space.getDimensions().getWidth();
		double height = space.getDimensions().getHeight();

		if (Math.abs(distantX) < width / 2) {
			space.moveByDisplacement(display, distantX / (ratio + 1), 0);
			space.moveByDisplacement(partner.display, -distantX / (1 / ratio + 1), 0);
		} else {
			distantX = (distantX > 0) ? distantX - width : distantX + width;
			space.moveByDisplacement(display, distantX / (ratio + 1), 0);
			space.moveByDisplacement(partner.display, -distantX / (1 / ratio + 1), 0);
		}

		if (Math.abs(distantY) < height / 2) {
			space.moveByDisplacement(display, 0, distantY / (ratio + 1));
			space.moveByDisplacement(partner.display, 0, -distantY / (1 / ratio + 1));
		} else {
			distantY = (distantY > 0) ? distantY - height : distantY + height;
			space.moveByDisplacement(display, 0, distantY / (ratio + 1));
			space.moveByDisplacement(partner.display, 0, -distantY / (1 / ratio + 1));
		}

		grid.moveTo(display, (int) space.getLocation(display).getX(), (int) space.getLocation(display).getY());
		grid.moveTo(partner.display, (int) space.getLocation(partner.display).getX(), (int) space.getLocation(partner.display).getY());
	}
	
	/**
	 * Return the ID of the agent.
	 * 
	 * @return		The ID of the agent.
	 */
	public int getID() {
		return ID;
	}

	/**
	 * Return the age of the agent.
	 * 
	 * @return		The age of the agent.
	 */
	public int getAge() {
		return age;
	}

	/**
	 * Return the gender of the agent. To be implemented by subclass
	 * 
	 * @return		The gender of the agent.
	 */
	public abstract String getGender();

	/**
	 * Return the spouse if the agent is married, otherwise return null.
	 * 
	 * @return		The spouse of the agent.
	 */
	public Human getPartner() {
		return partner;
	}
	
	/**
	 * Check if the agent is married.
	 * 
	 * @return		True if the agent is married. False otherwise.
	 */
	public boolean isMarried() {
		if (marDur > 0) return true;
		else return false;
	}

	/**
	 * Check if an agent is a relevant other of this agent.
	 * Depends upon age difference and age preference of this agent (as determined by midpoint)
	 * and also on distance between agents and the spatial distance parameter
	 * 
	 * @param 	relative	The agent to be checked.
	 * 
	 * @return	True if the agent is a relevant other. False otherwise.
	 */
	public boolean isARelevantOther(Human relative) {
		NdPoint pt = space.getLocation(display);
		NdPoint otherPt = space.getLocation(relative.display);


		double width = space.getDimensions().getWidth();
		double height = space.getDimensions().getHeight();
		if (((Math.abs(pt.getX() - otherPt.getX()) <= sd) || (Math.abs(pt.getX() - otherPt.getX()) >= width - sd))
				&& (isYAxisAge || (Math.abs(pt.getY() - otherPt.getY()) <= sd) || (Math.abs(pt.getY() - otherPt.getY()) >= height - sd))
				&& (relative.age >= age + midpoint - ageInterval) && (relative.age <= age + midpoint + ageInterval))
			return true;
		else return false;
	}

	/**
	 * Check if an agent is a potential partner of this agent. Does not check for gender.
	 * 
	 * @param 	partner		The agent to be checked.
	 * 
	 * @return	True if the agent is a potential partner. False otherwise.
	 */
	public boolean isAPotentialPartner(Human partner) {
		if (partner.isMarried()) return false;
		if (partner.age < 16) return false;
		NdPoint pt = space.getLocation(display);
		NdPoint otherPt = space.getLocation(partner.display);
		double width = space.getDimensions().getWidth();
		double height = space.getDimensions().getHeight();

		if (((Math.abs(pt.getX() - otherPt.getX()) <= distToSearch) || (Math.abs(pt.getX() - otherPt.getX()) >= width - distToSearch)) 
				&& (isYAxisAge || (Math.abs(pt.getY() - otherPt.getY()) <= distToSearch) || (Math.abs(pt.getY() - otherPt.getY()) >= height - distToSearch))
				&& (partner.age >= age - ageToSearch) && (partner.age <= age + ageToSearch))
			return true;
		else return false;
	}

	/**
	 * Do care transition. To be implemented by subclass.
	 */
	public abstract void careTransition();
}
