package WeddingGrid;

import java.util.ArrayList;

import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.graph.Network;
import repast.simphony.space.grid.Grid;
import repast.simphony.util.ContextUtils;
import repast.simphony.util.SimUtilities;

/**
 * A class that contain methods for Male agents only.
 * 
 * @author James
 *
 */
public class Male extends Human {
	public Male(Context<Object> context, ContinuousSpace<Object> space, Grid<Object> grid, int yearOfBirth, int age, boolean isYAxisAge) {
		super(context, space, grid, yearOfBirth, age, isYAxisAge);
		display = new MaleNormalStyle(this);
		context.add(display);
	}
	
	/**
	 * Find potential partners for the agent
	 */
	@Override
	public void findPotentialPartners() {
		potentialPartners = new ArrayList<Human>();
		
		Context<Object> context = ContextUtils.getContext(this);
		for (Object obj : context.getObjects(Female.class)) {
			if (isAPotentialPartner((Human) obj)) {
				potentialPartners.add((Human) obj);
			}			
		}
	}
	
	/**
	 * Return the gender of the agent
	 * 
	 * @return 		The gender
	 */
	public String getGender() {
		return "Male";
	}
	
	public boolean getHealthStatus() {
		return isIll;
	}
	
	/**
	 * Determine whether agent becomes ill. 
	 * Probability based on agents age 
	 * and  ageScaling, mainCareProb, baseCareProb parameters
	 * 
	 */
	@Override
	public void careTransition() {

		double ageScaling = (Double) RunEnvironment.getInstance().getParameters().getValue("ageScaling");
		double mainCareProb = (Double) RunEnvironment.getInstance().getParameters().getValue("mainCareProb");
		double baseCareProb = (Double) RunEnvironment.getInstance().getParameters().getValue("baseCareProb");
		double ageCareProb = ((Math.exp((double) age / (ageScaling + 1.0)) * (mainCareProb + 0.0001)));

		double careProb = baseCareProb + ageCareProb;

		if (Math.random() < careProb) {
			Context<Object> context = ContextUtils.getContext(this);
			Network<Object> net = (Network<Object>)context.getProjection("Married network");
			Object adj = net.getRandomAdjacent(display);
			NdPoint pt = space.getLocation(display);
			context.remove(display);
			display = new MaleInNeedStyle(this);
			context.add(display);
			space.moveTo(display, pt.getX(), pt.getY());
			grid.moveTo(display, (int) pt.getX(), (int) pt.getY());
			
			if (adj != null) {
				net.addEdge(display, adj);
				net.addEdge(adj, display);
			}
			isIll = true;
			setChanged();
			notifyObservers("ill");
		}
	}
}
