package WeddingGrid;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.lang.Math;

import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.util.ContextUtils;



/*
 * Class deals with recording to csv information required for outputs 
 */
public class StatisticCollector {

	private double[] hazardRate;
	private ArrayList<Double[]> decadeHazardRate;
	public ArrayList<Double> carerDist;
	private int count; //, set;
	/* note set variable can be adjusted manually to allow easier interpretation of outputs where multiple runs at different parameters are required.
	 *  see TheWorld, end of eachyear
	 */
	public static int runCount, lonelyCount, lonelyWidowed, lonelyKidsDead, set, caredByKids, caredByPartner;
	private CSVHandler hazardRecorder, humanRecorder, yearlyRecorder, decadeRecorder, mucmRecorder, yearlyCountRecorder, birthsRecorder, marriedBirthsRecorder, popRecorder,womenRecorder,marriedRecorder,marriageRecorder,carerRecorder;
	private String runType, time;
	public int width, height;
	int hazardPeak;
	//private String hazardRecord, humanRecord, yearlyRecord; 
	
	/* initiate local count variables */ 
	public StatisticCollector() {
		count = 1;
		set = 1;
		runCount = 1;
		lonelyCount = 0;
		lonelyWidowed =0;
		lonelyKidsDead=0;
		caredByKids=0;
		caredByPartner=0;
		carerDist = new ArrayList<Double>();
		hazardPeak=0;
	}

	
	/* set up recorder for hazard rates */
	public void newHazardRecord() {
		hazardRecorder = new CSVHandler(time + "_" + runType + "_hazard.rate.csv");		
		try {
			hazardRecorder.openFile(false);
			hazardRecorder.writeFieldTitles("Age", "Hazard rate");
			hazardRecorder.closeFile();
		} catch (Exception e) {}
		int maxAge = (Integer) RunEnvironment.getInstance().getParameters().getValue("maxAge");
		if (maxAge == 0) maxAge = Data.defaultMaxAge;
		this.hazardRate = new double[maxAge + 1];
	}
	
	/* setup new decade hazard rate collector - not currently used */ 
	public void newDecadeRecord() {
		decadeRecorder = new CSVHandler();
		decadeHazardRate = new ArrayList<Double[]>();
	}

	/* End of simulation recorder, now obsolete */
	public void newMucmRecord() {
		String name = "MUCM1_" + set  + "_" + runCount + ".csv";
		mucmRecorder = new CSVHandler(name);
		
		try {
			mucmRecorder.openFile(false);
			mucmRecorder.writeFieldTitles("alpha", "beta", "spatial distance", "total agents married","test agents ever married", "total agents", "married percentage", "run", "aiConstant", "MaxHaz");
			mucmRecorder.closeFile();
		} catch (Exception e) {}
	}
	
	/* creates new recorder for yearly count values, particularly for MUCM sensitivity and uncertainty analysis  */
	public void newYearlyCountRecord() {
		String name = "yearlyCount" + "_" + set  + "_" + runCount + ".csv";
		yearlyCountRecorder = new CSVHandler(name);
		try {
			yearlyCountRecorder.openFile(false);
			yearlyCountRecorder.writeFieldTitles("alpha", "beta", "Base Care Prob", "Single Care Prob", "Age Scaling", "Spatial Distance", "aiConstant", 
					"total agents", "married percentage", "lonely agents","lonely widow","lonely outlive kids", "lonely percentage","ill agents", "run", 
					"MarriedCount", "popsize", "caredByKids", "caredByPartner", "total16plus","MaxHaz");
			yearlyCountRecorder.closeFile();
		} catch (Exception e) {}
	}
	
	/* new recorder for individual specific information */
	public void newHumanRecord() {
		String name = time + "_" + runType + (RunEnvironment.getInstance().isBatch() ? "_" + count : "")  + "_human.record.csv";
		humanRecorder = new CSVHandler(name);
		//humanRecord = name;
		//humanRecorder.setPath(humanRecord);
		try {
			humanRecorder.openFile(false);
			humanRecorder.writeFieldTitles("ID", "Gender", "Birth date", "Death date", "Married date", "Spouse ID", "Gave birth in", "Children ID", "Agent type", "Dist To Search", "X", "Y");
			humanRecorder.closeFile();
		} catch (Exception e) {}
	}

	/* new very basic recorder for yearly events - not used in analysis.  */
	public void newYearlyRecord() {
		String name = time + "_" + runType + (RunEnvironment.getInstance().isBatch() ? "_" + count : "")  + "_yearly.record.csv";
		yearlyRecorder = new CSVHandler(name);
		try {
			yearlyRecorder.openFile(false);
			yearlyRecorder.writeFieldTitles("Year", "Number of birth", "Number of death");
			yearlyRecorder.closeFile();
		} catch (Exception e) {}
	}
	
	/* update partnership hazard rate */
	public void updateHazardRate(double[] hazardRate) {
		if (RunEnvironment.getInstance().isBatch()) {
			for (int i = 0; i < hazardRate.length; i++) {
				this.hazardRate[i] = (this.hazardRate[i] * (count - 1) + hazardRate[i]) / count; 	
			}
			count++;
		} else {
			this.hazardRate = hazardRate;
		}
		recordHazardData();
	}

	/* record partnership hazard rate data*/
	public void recordHazardData() {
		hazardRecorder.resetFile();
		try {
			hazardRecorder.openFile(true);
			for (int i = 0; i < hazardRate.length; i++) hazardRecorder.append(i, hazardRate[i]);
			hazardRecorder.closeFile();
		} catch (Exception e) {}
	}
	
	/* update decade partnership hazard rates */
	public void updateDecadeHazardRate(double[] hazardRate) {
		int tick = (int) RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
		int dc = tick / 10;
		if (decadeHazardRate.size() < dc) {
			
			int maxAge = (Integer) RunEnvironment.getInstance().getParameters().getValue("maxAge");
			if (maxAge == 0) maxAge = Data.defaultMaxAge;
			decadeHazardRate.add(new Double[maxAge + 1]);
		}
		for (int i = 0; i < decadeHazardRate.get(dc - 1).length; i++) {
			if (decadeHazardRate.get(dc - 1)[i] == null) decadeHazardRate.get(dc - 1)[i] = new Double(0);
			decadeHazardRate.get(dc - 1)[i] = (decadeHazardRate.get(dc - 1)[i] * (count - 1) + hazardRate[i]) / count; 
		}
		recordDecadeHazardData();
	}
	
	/* record decade partnership hazard rates */
	public void recordDecadeHazardData() {
		int tick = (int) RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
		
		int dc = tick / 10;
		try {
			String decadeFileName = time + "_" + runType + "_hazard.rate_" + (1950 + tick - 9) + "-" + (1950 + tick) + ".csv";
			decadeRecorder.setPath(decadeFileName);
			decadeRecorder.openFile(false);
			decadeRecorder.writeFieldTitles("Age", "Hazard rate");
			for (int i = 0; i < decadeHazardRate.get(dc - 1).length; i++) decadeRecorder.append(i, decadeHazardRate.get(dc - 1)[i]);
			decadeRecorder.closeFile();
		} catch (Exception e) {
		}
	}

	public void recordHuman(Human human) {

		try {
			humanRecorder.openFile(true);
			Integer[] childrenIDs = new Integer[human.children.size()];
			Integer[] childrenBirthDate = new Integer[human.children.size()];
			for (int i = 0; i < human.children.size(); i++) {
				childrenIDs[i] = human.children.get(i).ID;
				childrenBirthDate[i] = human.children.get(i).yearOfBirth;
			}
			String partnerID = "";
			if (human.partner != null) partnerID = ((Integer) human.partner.ID).toString();
			humanRecorder.append(human.ID, human.getGender(), human.yearOfBirth, human.yearOfDeath, human.yearOfMarriage, partnerID, humanRecorder.toCSVField(childrenBirthDate), humanRecorder.toCSVField(childrenIDs), human.type, human.distToSearch, human.getX(), human.getY());
			humanRecorder.closeFile();
		} catch (Exception e) {}
	}
	

	/* function that check whether and individual has available relatives to care for them, and updates counts reflecting their care status */ 
	public void checkLonely(Human human) {	
		int numUnavailableChildren=0;
		if (human.isIll == false){return;}  
		if (human.partner == null) {
			lonelyCount++;
		} else if (human.partner.yearOfDeath == 0 && human.partner.isIll==false)  {
			caredByPartner++;
		} else if (human.children.size() ==0){
			lonelyCount++;
			lonelyWidowed++;
		} else {
			for (Human child : human.children){
				if (child.yearOfDeath!=0 || child.isIll|| child.age<16){
					numUnavailableChildren++;
				}
			}
			if (numUnavailableChildren== human.children.size()){
				lonelyCount++;
				lonelyKidsDead++;
			} else caredByKids++;
		}
	}
	
	/* function that records the distance to an ill person's nearest carer. Called once per sim. 
	 * gives 0 if cared for by co-resident partner, and 999 if there is no available carer 
	 */ 
	public void checkCarer(Human human) {
		double distanceToCarer=999;
		double dist = 999;
		width = (Integer) RunEnvironment.getInstance().getParameters().getValue("spaceWidth");
		height = (Integer) RunEnvironment.getInstance().getParameters().getValue("spaceHeight");
		if (human.isIll == true){
			if (human.partner == null) {
			} else if (human.partner.yearOfDeath == 0 && human.partner.isIll==false) {distanceToCarer=0;
			} else if (human.children.size() ==0){
			} else {
				for (Human child : human.children){
					if (child.yearOfDeath==0 && child.isIll==false && child.age>15){
						double thisx=human.getX();
						double thisy=human.getY();
						double carerx=child.getX();
						double carery=child.getY();
						double xDistanceSq = java.lang.Math.pow(java.lang.Math.min(java.lang.Math.abs(thisx-carerx),width-java.lang.Math.abs(carerx-thisx)),2);
						double yDistanceSq = java.lang.Math.pow(java.lang.Math.min(java.lang.Math.abs(thisy-carery),height-java.lang.Math.abs(carery-thisy)),2);
						dist = java.lang.Math.sqrt(xDistanceSq+yDistanceSq);
						if (dist < distanceToCarer){
							distanceToCarer=dist;
						}
					}
				}
			}
			this.carerDist.add(distanceToCarer);
		}
	} 
	
    
	

	//now-obsolete end of simulation count recorder
	public void recordMucm(int marriageCountTotal, int agentTotal, int testMarriageCount) {
		double oldMarriedPercentage = (((double) testMarriageCount)/((double) agentTotal)) * 100;
		try {
			mucmRecorder.openFile(true);
			mucmRecorder.append(RunEnvironment.getInstance().getParameters().getValue("alpha"), RunEnvironment.getInstance().getParameters().getValue("beta"), RunEnvironment.getInstance().getParameters().getValue("sd"), marriageCountTotal, testMarriageCount, agentTotal, oldMarriedPercentage, runCount,RunEnvironment.getInstance().getParameters().getValue("AIConstant"),this.hazardPeak);
			mucmRecorder.closeFile();
		} catch (Exception e) {}
	} 
	
	//record yearly counts data and run paramters. Calculate and record percentages for percents lonely and married 
	public void recordYearlyCounts(int marriageCountTotal, int illCount, int agentTotal,int popsize, int total16plus) {
		double marriedPercentage = (((double) marriageCountTotal)/((double) total16plus)) * 100;
		double lonelyPercentage = (((double) lonelyCount)/((double) popsize)) * 100;
		try {
			yearlyCountRecorder.openFile(true);
			yearlyCountRecorder.append(   RunEnvironment.getInstance().getParameters().getValue("alpha"), 
									RunEnvironment.getInstance().getParameters().getValue("beta"),	
									RunEnvironment.getInstance().getParameters().getValue("baseCareProb"), 
									RunEnvironment.getInstance().getParameters().getValue("mainCareProb"), 
									RunEnvironment.getInstance().getParameters().getValue("ageScaling"), 
									RunEnvironment.getInstance().getParameters().getValue("sd"),
									RunEnvironment.getInstance().getParameters().getValue("AIConstant"),
									agentTotal, marriedPercentage, lonelyCount,lonelyWidowed, lonelyKidsDead, lonelyPercentage, 
									illCount, runCount, marriageCountTotal, popsize,caredByKids, caredByPartner, total16plus, this.hazardPeak); 
			yearlyCountRecorder.closeFile();
		} catch (Exception e) {}
		
	}
	
	/* Basic event per year recorder. No longer used for analysis */
	public void recordPerYear(Object... obj) {
		try {
			yearlyRecorder.openFile(true);
			yearlyRecorder.append(obj);
			yearlyRecorder.closeFile();
		} catch (Exception e) {}
	}

	public int getCount() {
		return count;
	}

	public void resetCount() {
		count = 1;
	}

	public void setRunType() {
		if(RunEnvironment.getInstance().isBatch()) {
			runType = "batch";
		} else {
			runType = "single";
		}
	}

	public void setTime() {
		Calendar c = Calendar.getInstance();
		time = c.get(Calendar.YEAR) + "." + (c.get(Calendar.MONTH) + 1) + "." + c.get(Calendar.DAY_OF_MONTH) 
			   + "_"
			   + c.get(Calendar.HOUR_OF_DAY) + "." + c.get(Calendar.MINUTE) + "." + c.get(Calendar.SECOND);  
	}

	
	/*
	 * Below follows a long list of functions to record population and event numbers by year and age. 
	 * 
	 */
	
	public void recordBirthStats(int[][] birthsByAgeAndYear) {
		String name = "BirthStats_" + set  + "_" + runCount + ".csv";
		birthsRecorder = new CSVHandler(name);
		
		try {
			birthsRecorder.openFile(false);
			birthsRecorder.write("alpha",RunEnvironment.getInstance().getParameters().getValue("alpha"),"beta", RunEnvironment.getInstance().getParameters().getValue("beta"),"sd", RunEnvironment.getInstance().getParameters().getValue("sd"));
			for (int i= 0;i<birthsByAgeAndYear.length; i++){
				String line = ""; 
				for (int j= 0; j<birthsByAgeAndYear[i].length; j++){
					line=line + String.valueOf(birthsByAgeAndYear[i][j]) + "," ;
				}
				birthsRecorder.append(line);
			}
			birthsRecorder.closeFile();
		} catch (Exception e) {}
	}
	
	public void recordMarriedBirthStats(int[][] marriedBirthsByAgeAndYear) {
		String name = "marriedBirthStats_" + set  + "_" + runCount + ".csv";
		marriedBirthsRecorder = new CSVHandler(name);
		
		try {
			marriedBirthsRecorder.openFile(false);
			marriedBirthsRecorder.write("alpha",RunEnvironment.getInstance().getParameters().getValue("alpha"),
					"beta", RunEnvironment.getInstance().getParameters().getValue("beta"),"sd", RunEnvironment.getInstance().getParameters().getValue("sd"));
			for (int i= 0;i<marriedBirthsByAgeAndYear.length; i++){
				String line = ""; 
				for (int j= 0; j<marriedBirthsByAgeAndYear[i].length; j++){
					line=line + String.valueOf(marriedBirthsByAgeAndYear[i][j]) + "," ;
				}
				marriedBirthsRecorder.append(line);
			}
			marriedBirthsRecorder.closeFile();
		} catch (Exception e) {System.out.println(e);}
	}
	
	

	public void recordPopStats(int[][] humanByAgeAndYear) {
		String name = "PopStats_" + set  + "_" + runCount + ".csv";
		popRecorder = new CSVHandler(name);
		
		try {
			popRecorder.openFile(false);
			popRecorder.write("alpha",RunEnvironment.getInstance().getParameters().getValue("alpha"),"beta", RunEnvironment.getInstance().getParameters().getValue("beta"),"sd", RunEnvironment.getInstance().getParameters().getValue("sd"));
			for (int i= 0;i<humanByAgeAndYear.length; i++){
				String line = ""; 
				for (int j= 0; j<humanByAgeAndYear[i].length; j++){
					line=line + String.valueOf(humanByAgeAndYear[i][j]) + "," ;
				}
				popRecorder.append(line);
			}
			popRecorder.closeFile();
		} catch (Exception e) {}
		
	}

	public void recordWomenStats(int[][] womenByAgeAndYear) {
		String name = "WomenPopStats_" + set  + "_" + runCount + ".csv";
		womenRecorder = new CSVHandler(name);
		
		try {
			womenRecorder.openFile(false);
			womenRecorder.write("alpha",RunEnvironment.getInstance().getParameters().getValue("alpha"),"beta", RunEnvironment.getInstance().getParameters().getValue("beta"),"sd", RunEnvironment.getInstance().getParameters().getValue("sd"));
			for (int i= 0;i<womenByAgeAndYear.length; i++){
				String line = ""; 
				for (int j= 0; j<womenByAgeAndYear[i].length; j++){
					line=line + String.valueOf(womenByAgeAndYear[i][j]) + "," ;
				}
				womenRecorder.append(line);
			}
			womenRecorder.closeFile();
		} catch (Exception e) {}
		
	}

	public void recordMarriedStats(int[][] marriedWomenByAgeAndYear) {
		String name = "marriedPopStats_" + set  + "_" + runCount + ".csv";
		marriedRecorder = new CSVHandler(name);
		
		try {
			marriedRecorder.openFile(false);
			marriedRecorder.write("alpha",RunEnvironment.getInstance().getParameters().getValue("alpha"),"beta", RunEnvironment.getInstance().getParameters().getValue("beta"),"sd", RunEnvironment.getInstance().getParameters().getValue("sd"));
			for (int i= 0;i<marriedWomenByAgeAndYear.length; i++){
				String line = ""; 
				for (int j= 0; j<marriedWomenByAgeAndYear[i].length; j++){
					line=line + String.valueOf(marriedWomenByAgeAndYear[i][j]) + "," ;
				}
				marriedRecorder.append(line);
			}
			marriedRecorder.closeFile();
		} catch (Exception e) {}
		
	}
	
	public void recordCarerStats(ArrayList<Double> careDist) {
		String name = "carerStats_" + set  + "_" + runCount + ".csv";
		carerRecorder = new CSVHandler(name);
		try {
			carerRecorder.openFile(false);
			carerRecorder.write("mainCareProb",RunEnvironment.getInstance().getParameters().getValue("mainCareProb"),"baseCareProb", RunEnvironment.getInstance().getParameters().getValue("baseCareProb"));
			String line = ""; 
				for (int i= 0; i<carerDist.size(); i++){
					line=line + String.valueOf(carerDist.get(i)) + "," ;
				}
				carerRecorder.append(line);
			carerRecorder.closeFile();
		} catch (Exception e) {}
	}
	

	public void recordMarriageStats(int[][] marriagesByAgeAndYear) {
		String name = "marriageStats_" + set  + "_" + runCount + ".csv";
		marriageRecorder = new CSVHandler(name);
		try {
			marriageRecorder.openFile(false);
			marriageRecorder.write("alpha",RunEnvironment.getInstance().getParameters().getValue("alpha"),"beta", RunEnvironment.getInstance().getParameters().getValue("beta"),"sd", RunEnvironment.getInstance().getParameters().getValue("sd"));
			for (int i= 0;i<marriagesByAgeAndYear.length; i++){
				String line = ""; 
				for (int j= 0; j<marriagesByAgeAndYear[i].length; j++){
					line=line + String.valueOf(marriagesByAgeAndYear[i][j]) + "," ;
				}
				marriageRecorder.append(line);
			}
			marriageRecorder.closeFile();
		} catch (Exception e) {}
		
	}
}
